
LOG_RECORD_DEFAULT_ATTRIBUTES = [
    'args', 'created', 'exc_info', 'exc_text', 'filename',
    'funcName', 'levelname', 'levelno', 'lineno', 'module',
    'msecs', 'msg', 'name', 'pathname', 'process', 'processName',
    'relativeCreated', 'stack_info', 'thread', 'threadName'
]

SOCKET_RETRY_TIMEOUT = 1
