#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aio_logstash']

package_data = \
{'': ['*']}

setup(name='aio-logstash',
      version='0.1.0',
      description='Python AsyncIO Logstash Logger',
      author='Sina Khorami',
      author_email='sinakhorami94@gmail.com',
      url='https://github.com/SinaKhorami/aio-logstash',
      packages=packages,
      package_data=package_data,
      python_requires='>=3.7',
     )
